@@ -14,10 +14,12 @@
 #include "broker/internal/prometheus.hh"
 #include "broker/internal/type_id.hh"
 #include "broker/internal/web_socket.hh"
+#include "broker/port.hh"
 #include "broker/publisher.hh"
 #include "broker/status_subscriber.hh"
 #include "broker/subscriber.hh"
 #include "broker/timeout.hh"
+#include "broker/zeek.hh"
 
 #include <caf/actor.hpp>
 #include <caf/actor_system.hpp>
@@ -566,11 +568,12 @@ endpoint::endpoint(configuration config, endpoint_id id) : id_(id) {
   }
   core_ = facade(core);
   // Spin up a Prometheus actor if configured or an exporter.
-  if (auto port = caf::get_as<uint16_t>(cfg, "broker.metrics.port")) {
+  if (auto port = caf::get_as<broker::port>(cfg, "broker.metrics.port")) {
     auto ptask = std::make_unique<prometheus_http_task>(sys);
     auto addr = caf::get_or(cfg, "broker.metrics.address", std::string{});
-    if (auto actual_port = ptask->start(
-          *port, native(core_), addr.empty() ? nullptr : addr.c_str())) {
+    if (auto actual_port =
+          ptask->start(port->number(), native(core_),
+                       addr.empty() ? nullptr : addr.c_str())) {
       BROKER_INFO("expose metrics on port" << *actual_port);
       telemetry_exporter_ = facade(ptask->telemetry_exporter());
       background_tasks_.emplace_back(std::move(ptask));
@@ -584,8 +587,9 @@ endpoint::endpoint(configuration config, endpoint_id id) : id_(id) {
     telemetry_exporter_ = facade(hdl);
   }
   // Spin up a WebSocket server when requested.
-  if (auto port = caf::get_as<uint16_t>(cfg, "broker.web-socket.port"))
-    web_socket_listen("0.0.0.0"s, *port);
+  if (auto port = caf::get_as<broker::port>(cfg, "broker.web-socket.port"))
+    web_socket_listen(caf::get_or(cfg, "broker.web-socket.address", ""s),
+                      port->number());
 }
 
 endpoint::~endpoint() {
@@ -807,17 +811,41 @@ void endpoint::forward(std::vector<topic> ts) {
 }
 
 void endpoint::publish(topic t, data d) {
-  BROKER_INFO("publishing" << std::make_pair(t, d));
+  BROKER_INFO("publishing" << d << "at" << t);
   caf::anon_send(native(core_), atom::publish_v,
                  make_data_message(std::move(t), std::move(d)));
 }
 
+void endpoint::publish(topic t, variant d) {
+  BROKER_INFO("publishing" << d << "at" << t);
+  caf::anon_send(native(core_), atom::publish_v,
+                 make_data_message(std::move(t), std::move(d)));
+}
+
+void endpoint::publish(std::string_view t, const zeek::Message& d) {
+  BROKER_INFO("publishing" << d << "at" << t);
+  caf::anon_send(native(core_), atom::publish_v, make_data_message(t, d.raw()));
+}
+
 void endpoint::publish(const endpoint_info& dst, topic t, data d) {
-  BROKER_INFO("publishing" << std::make_pair(t, d) << "to" << dst.node);
+  BROKER_INFO("publishing" << d << "at" << t << "to" << dst.node);
   caf::anon_send(native(core_), atom::publish_v,
                  make_data_message(std::move(t), std::move(d)), dst);
 }
 
+void endpoint::publish(const endpoint_info& dst, topic t, const variant& d) {
+  BROKER_INFO("publishing" << d << "at" << t << "to" << dst.node);
+  caf::anon_send(native(core_), atom::publish_v,
+                 make_data_message(std::move(t), std::move(d)), dst);
+}
+
+void endpoint::publish(const endpoint_info& dst, std::string_view t,
+                       const zeek::Message& d) {
+  BROKER_INFO("publishing" << d << "at" << t << "to" << dst.node);
+  caf::anon_send(native(core_), atom::publish_v, make_data_message(t, d.raw()),
+                 dst);
+}
+
 void endpoint::publish(data_message x) {
   BROKER_INFO("publishing" << x);
   caf::anon_send(native(core_), atom::publish_v, std::move(x));
