@@ -77,7 +77,7 @@ private:
  *
  * Value instances are tied to a `Parent` instance. The value's data will
  * remain accessible only as long as the parent stays around. If that goes
- * away, deferencing will throw an error.
+ * away, dereferencing will throw an error.
  */
 class Value {
 public:
@@ -378,6 +378,68 @@ class Address : public detail::AtomicType<hilti::rt::Address> {};
 /** Type information for type ``any`. */
 class Any : public detail::ValueLessType {};
 
+class Bitfield;
+
+namespace bitfield {
+
+/** Auxiliary type information for type ``bitfield`` describing one of its fields. */
+class Bits {
+public:
+    /**
+     * Constructor.
+     *
+     * @param name ID of the field
+     * @param type type of the field
+     * @param offset offset of the field inside the bitfield's storage tuple.
+     */
+    Bits(const char* name, const TypeInfo* type, std::ptrdiff_t offset) : name(name), type(type), offset(offset) {}
+
+    const std::string name; /**< ID of the field, with an empty string indicating no name */
+    const TypeInfo* type;   /**< type of the field */
+
+private:
+    friend class type_info::Bitfield;
+
+    const std::ptrdiff_t offset;
+};
+
+}; // namespace bitfield
+
+/** Auxiliary type information for type ``bitfield`. */
+class Bitfield {
+public:
+    /**
+     * Constructor
+     *
+     * @param labels the bitfield's fields
+     */
+    Bitfield(std::vector<bitfield::Bits> bits) : _bits(std::move(bits)) {}
+
+    /** Returns the bitfield's fields. */
+    const auto& bits() const { return _bits; }
+
+    /**
+     * Returns a vector that can be iterated over to visit all the fields.
+     *
+     * @param v the value referring to the bitfield to iterate over
+     *
+     * @return a vector of pairs ``(field, value)`` where *field* is the
+     * current ``bitfield::Bit` and *value* is the field's value.
+     */
+    auto iterate(const Value& v) const {
+        std::vector<std::pair<const bitfield::Bits&, Value>> values;
+
+        values.reserve(_bits.size());
+        for ( const auto& f : _bits )
+            values.emplace_back(f, Value(static_cast<const char*>(v.pointer()) + f.offset, f.type, v));
+
+        return values;
+    }
+
+private:
+    const std::vector<bitfield::Bits> _bits;
+};
+
 /** Type information for type ``bool`. */
 class Bool : public detail::AtomicType<bool> {};
 
@@ -573,7 +635,7 @@ public:
     const TypeInfo* keyType() const { return _ktype; }
 
     /**
-     * Returns the type of the valye of the elements, as passed into the constructor.
+     * Returns the type of the value of the elements, as passed into the constructor.
      */
     const TypeInfo* valueType() const { return _vtype; }
 
@@ -680,7 +742,7 @@ public:
     const TypeInfo* keyType() const { return _ktype; }
 
     /**
-     * Returns the type of the valye of the elements, as passed into the constructor.
+     * Returns the type of the value of the elements, as passed into the constructor.
      */
     const TypeInfo* valueType() const { return _vtype; }
 
@@ -838,9 +900,9 @@ struct Field {
      * @param offset offset of the field in number bytes inside the struct
      * @param accessor function returning a pointer to a fields value
      */
-    Field(const char* name, const TypeInfo* type, std::ptrdiff_t offset, bool internal,
+    Field(const char* name, const TypeInfo* type, std::ptrdiff_t offset, bool internal, bool anonymous,
           Accessor accessor = accessor_default)
-        : name(name), type(type), offset(offset), accessor(accessor), internal(internal) {}
+        : name(name), type(type), offset(offset), accessor(accessor), internal(internal), anonymous(anonymous) {}
 
     /** Default accessor function suitable for non-optional fields. */
     static const void* accessor_default(const Value& v) { return v.pointer(); }
@@ -859,6 +921,7 @@ struct Field {
         };
     }
 
+    bool isAnonymous() const { return anonymous; }
     bool isInternal() const { return internal; }
 
     const std::string name; /**< ID of the field */
@@ -873,6 +936,7 @@ private:
     const std::ptrdiff_t offset;
     const Accessor accessor;
     const bool internal;
+    const bool anonymous;
 };
 
 }; // namespace struct_
@@ -1148,6 +1212,7 @@ struct TypeInfo {
         Undefined,
         Address,
         Any,
+        Bitfield,
         Bool,
         Bytes,
         BytesIterator,
@@ -1198,6 +1263,7 @@ struct TypeInfo {
     union {
         type_info::Address* address;
         type_info::Any* any;
+        type_info::Bitfield* bitfield;
         type_info::Bool* bool_;
         type_info::Bytes* bytes;
         type_info::BytesIterator* bytes_iterator;
@@ -1256,6 +1322,10 @@ struct TypeInfo {
             tag = Any;
             any = value;
         }
+        else if constexpr ( std::is_same_v<Type, type_info::Bitfield> ) {
+            tag = Bitfield;
+            bitfield = value;
+        }
         else if constexpr ( std::is_same_v<Type, type_info::Bool> ) {
             tag = Bool;
             bool_ = value;
@@ -1450,6 +1520,10 @@ const Type* auxType(const type_info::Value& v) {
         assert(type.tag == TypeInfo::Any);
         return type.any;
     }
+    else if constexpr ( std::is_same_v<Type, type_info::Bitfield> ) {
+        assert(type.tag == TypeInfo::Bitfield);
+        return type.bitfield;
+    }
     else if constexpr ( std::is_same_v<Type, type_info::Bool> ) {
         assert(type.tag == TypeInfo::Bool);
         return type.bool_;
