@@ -81,6 +81,52 @@ static void ZeekGlobalVarsEnumerator(const v8::PropertyCallbackInfo<v8::Array>&
   info.GetReturnValue().Set(array);
 }
 
+void ZeekGlobalVarsSetter(v8::Local<v8::Name> property,
+                          v8::Local<v8::Value> v8_val,
+                          const v8::PropertyCallbackInfo<v8::Value>& info) {
+  v8::Isolate* isolate = info.GetIsolate();
+  auto zeek_obj = v8::Local<v8::Object>::Cast(info.Data());
+  auto field = v8::Local<v8::External>::Cast(zeek_obj->GetInternalField(0));
+  auto instance = static_cast<Instance*>(field->Value());
+
+  v8::String::Utf8Value arg(isolate, property);
+  dprintf("Property... %s", *arg);
+
+  if (!*arg)
+    return;
+
+  const zeek::detail::IDPtr& id = zeek::id::find(*arg);
+
+  // Don't allow the obvious ones that won't work.
+  if (!id || id->IsType() || id->IsConst() || id->IsOption() || !id->GetType()) {
+    std::string what;
+    if (!id)
+      what = "unknown global";
+    else if (id->IsType())
+      what = "type";
+    else if (id->IsConst())
+      what = "const";
+    else if (id->IsOption())
+      what = "option";
+    else if (!id->GetType())
+      what = "without type";
+
+    std::string error = zeek::util::fmt("Cannot set %s: %s", what.c_str(), *arg);
+    isolate->ThrowException(v8_str(isolate, error.c_str()));
+    return;
+  }
+
+  ZeekValWrapper::Result wrap_result = instance->ToZeekVal(v8_val, id->GetType());
+  if (!wrap_result.ok) {
+    v8::Local<v8::Value> error = v8::Exception::TypeError(
+        ::v8_str(isolate, zeek::util::fmt("Bad value: %s", wrap_result.error.c_str())));
+    isolate->ThrowException(error);
+    return;
+  }
+
+  id->SetVal(wrap_result.val);
+}
+
 // Call a Javascript function with Zeek land arguments.
 //
 // The caller is supposed to enter the IsolateScope and HandleScope so that
@@ -469,8 +515,8 @@ zeek::TypePtr try_name_to_basetype(const std::string& name) {
   for (int i = 0; i <= zeek::TYPE_ERROR; i++) {
     auto tag = static_cast<zeek::TypeTag>(i);
     if (name == zeek::type_name(tag)) {
-      return zeek::base_type(tag);
-      break;
+      if (const auto& t = zeek::base_type(tag); zeek::is_atomic_type(t))
+        return t;
     }
   }
   return nullptr;
@@ -486,12 +532,15 @@ v8::MaybeLocal<v8::Value> Instance::ZeekAs(v8::Local<v8::String> v8_name,
 
   if (!id)
     as_type = try_name_to_basetype(name);
-  else
+  else if (id->IsType())
     as_type = id->GetType();
 
+  if (!as_type)
+    as_type = zeek_type_registry_->Lookup(name);
+
   if (!as_type) {
-    isolate_->ThrowException(
-        v8_str(isolate_, zeek::util::fmt("'%s' is not a Zeek type", name.c_str())));
+    isolate_->ThrowException(v8_str(
+        isolate_, zeek::util::fmt("cannot find Zeek type for '%s'", name.c_str())));
     return {};
   }
 
@@ -695,17 +744,16 @@ void Instance::AddZeekObject(v8::Local<v8::Object> exports,
   // global_vars dictionary
   v8::Local<v8::String> globals_str = v8_str_intern(isolate, "global_vars");
   v8::Local<v8::ObjectTemplate> zeek_globals_tmpl = v8::ObjectTemplate::New(isolate);
-  zeek_globals_tmpl->SetInternalFieldCount(1);
 
   v8::NamedPropertyHandlerConfiguration global_vars_conf = {nullptr};
   global_vars_conf.getter = ZeekGlobalVarsGetter;
   global_vars_conf.enumerator = ZeekGlobalVarsEnumerator;
+  global_vars_conf.setter = ZeekGlobalVarsSetter;
   global_vars_conf.data = zeek_obj;
   zeek_globals_tmpl->SetHandler(global_vars_conf);
 
   v8::Local<v8::Object> zeek_global_vars_obj =
       zeek_globals_tmpl->NewInstance(context).ToLocalChecked();
-  zeek_global_vars_obj->SetInternalField(0, v8::External::New(isolate, instance));
   zeek_obj->Set(context, globals_str, zeek_global_vars_obj).Check();
 
   // Files to be loaded by the bootstrapping script.
@@ -919,6 +967,7 @@ bool Instance::Init(plugin::Corelight_ZeekJS::Plugin* plugin,
       &node::FreeEnvironment};
 
   zeek_val_wrapper_ = std::make_unique<ZeekValWrapper>(GetIsolate());
+  zeek_type_registry_ = std::make_unique<ZeekTypeRegistry>();
 
   node::AddLinkedBinding(node_environment_.get(), "zeekjs", RegisterModule, this);
 
