@@ -8,6 +8,7 @@
 
 #include <hilti/ast/all.h>
 #include <hilti/ast/builder/all.h>
+#include <hilti/ast/builder/expression.h>
 #include <hilti/base/preprocessor.h>
 #include <hilti/base/util.h>
 #include <hilti/compiler/unit.h>
@@ -988,7 +989,8 @@ bool GlueCompiler::compile() {
     if ( ! _doc_id.empty() ) {
         auto mtime = hilti::expression::Ctor(hilti::ctor::Time(hilti::rt::time::current_time()));
         preinit_body.addCall("zeek_rt::register_spicy_module_begin",
-                             {hilti::builder::string(_doc_id), hilti::builder::string(_doc_description), mtime});
+                             {hilti::builder::string_mut(_doc_id), hilti::builder::string_mut(_doc_description),
+                              mtime});
     }
 
     for ( auto& a : _protocol_analyzers ) {
@@ -1027,14 +1029,14 @@ bool GlueCompiler::compile() {
 
         preinit_body.addCall(
             "zeek_rt::register_protocol_analyzer",
-            {builder::string(a.name), builder::id(protocol),
+            {builder::string_mut(a.name), builder::id(protocol),
              builder::vector(hilti::util::transform(
                  a.ports,
                  [](const auto& p) {
                      return builder::call("zeek_rt::make_port_range", {builder::port(p.begin), builder::port(p.end)});
                  })),
-             builder::string(a.unit_name_orig), builder::string(a.unit_name_resp), builder::string(a.replaces),
-             _linker_scope()});
+             builder::string_mut(a.unit_name_orig), builder::string_mut(a.unit_name_resp),
+             builder::string_mut(a.replaces), _linker_scope()});
     }
 
     for ( auto& a : _file_analyzers ) {
@@ -1050,10 +1052,10 @@ bool GlueCompiler::compile() {
         }
 
         preinit_body.addCall("zeek_rt::register_file_analyzer",
-                             {builder::string(a.name),
+                             {builder::string_mut(a.name),
                               builder::vector(
-                                  hilti::util::transform(a.mime_types, [](auto m) { return builder::string(m); })),
-                              builder::string(a.unit_name), builder::string(a.replaces), _linker_scope()});
+                                  hilti::util::transform(a.mime_types, [](auto m) { return builder::string_mut(m); })),
+                              builder::string_mut(a.unit_name), builder::string_mut(a.replaces), _linker_scope()});
     }
 
     for ( auto& a : _packet_analyzers ) {
@@ -1069,8 +1071,8 @@ bool GlueCompiler::compile() {
         }
 
         preinit_body.addCall("zeek_rt::register_packet_analyzer",
-                             {builder::string(a.name), builder::string(a.unit_name), builder::string(a.replaces),
-                              _linker_scope()});
+                             {builder::string_mut(a.name), builder::string_mut(a.unit_name),
+                              builder::string_mut(a.replaces), _linker_scope()});
     }
 
     // Create the Spicy hooks and accessor functions.
@@ -1081,15 +1083,14 @@ bool GlueCompiler::compile() {
 
     // Register our Zeek events at pre-init time.
     for ( auto&& ev : _events )
-        preinit_body.addCall("zeek_rt::install_handler", {builder::string(ev.name)});
+        preinit_body.addCall("zeek_rt::install_handler", {builder::string_mut(ev.name)});
 
     // Create Zeek types for exported Spicy types.
+    GlueCompiler::ZeekTypeCache cache;
+
     for ( const auto& [tinfo, id] : _driver->exportedTypes() ) {
-        if ( auto type = createZeekType(tinfo.type, id) )
-            preinit_body.addCall("zeek_rt::register_type",
-                                 {builder::string(id.namespace_()), builder::string(id.local()), *type});
-        else
-            hilti::logger().error(hilti::util::fmt("cannot export Spicy type '%s': %s", id, type.error()),
+        if ( auto rc = createZeekType(tinfo.type, id, &preinit_body, &cache); ! rc )
+            hilti::logger().error(hilti::util::fmt("cannot export Spicy type '%s': %s", id, rc.error()),
                                   tinfo.location);
     }
 
@@ -1239,8 +1240,9 @@ bool GlueCompiler::CreateSpicyHook(glue::Event* ev) {
 
     // Define Zeek-side event handler.
     auto handler_id = hilti::ID(hilti::util::fmt("__zeek_handler_%s", mangled_event_name));
-    auto handler = builder::global(handler_id, builder::call("zeek_rt::internal_handler", {builder::string(ev->name)}),
-                                   hilti::declaration::Linkage::Private, meta);
+    auto handler =
+        builder::global(handler_id, builder::call("zeek_rt::internal_handler", {builder::string_mut(ev->name)}),
+                        hilti::declaration::Linkage::Private, meta);
     ev->spicy_module->spicy_module->add(std::move(handler));
 
     // Create the hook body that raises the event.
@@ -1269,11 +1271,11 @@ bool GlueCompiler::CreateSpicyHook(glue::Event* ev) {
     // Zeek type to convert an Spicy type into. However, we wouldn't want
     // limit logging to events with handlers.
     if ( _driver->hiltiOptions().debug ) {
-        std::vector<hilti::Expression> fmt_args = {builder::string(ev->name)};
+        std::vector<hilti::Expression> fmt_args = {builder::string_literal(ev->name.str())};
 
         for ( const auto&& [i, e] : hilti::util::enumerate(ev->expression_accessors) ) {
             if ( hilti::util::startsWith(e.expression, "$") ) {
-                fmt_args.emplace_back(builder::string(e.expression));
+                fmt_args.emplace_back(builder::string_literal(e.expression));
                 continue;
             }
 
@@ -1281,12 +1283,12 @@ bool GlueCompiler::CreateSpicyHook(glue::Event* ev) {
                 fmt_args.emplace_back(std::move(*expr));
             else
                 // We'll catch and report this below.
-                fmt_args.emplace_back(builder::string("<error>"));
+                fmt_args.emplace_back(builder::string_literal("<error>"));
         }
 
         std::vector<std::string> fmt_ctrls(fmt_args.size() - 1, "%s");
         auto fmt_str = hilti::util::fmt("-> event %%s(%s)", hilti::util::join(fmt_ctrls, ", "));
-        auto msg = builder::modulo(builder::string(fmt_str), builder::tuple(fmt_args));
+        auto msg = builder::modulo(builder::string_literal(fmt_str), builder::tuple(fmt_args));
         auto call = builder::call("zeek_rt::debug", {std::move(msg)});
         body.addExpression(call);
     }
@@ -1358,9 +1360,12 @@ namespace {
 // Note: Any logic changes here must be reflected in the plugin driver's
 // corresponding `VisitorZeekType` as well.
 struct VisitorZeekType : hilti::visitor::PreOrder<hilti::Result<hilti::Expression>, VisitorZeekType> {
-    VisitorZeekType(const GlueCompiler* gc) : gc(gc) {}
+    VisitorZeekType(const GlueCompiler* gc, hilti::builder::Builder* body, GlueCompiler::ZeekTypeCache* cache)
+        : gc(gc), body(body), cache(cache) {}
 
     const GlueCompiler* gc;
+    hilti::builder::Builder* body;
+    GlueCompiler::ZeekTypeCache* cache;
 
     std::set<hilti::ID> zeek_types;
     std::vector<std::optional<hilti::ID>> ids = {};
@@ -1373,6 +1378,11 @@ struct VisitorZeekType : hilti::visitor::PreOrder<hilti::Result<hilti::Expressio
         return (! ids.empty() && ids.front().has_value()) ? ids.front()->namespace_() : hilti::ID();
     }
 
+    // Returns prefix for a new tmp that includes the given ID in its name.
+    auto tmpName(const std::string& prefix, const hilti::ID& id) const {
+        return hilti::util::fmt("%s_%s", prefix, hilti::util::replace(id, "::", "_"));
+    }
+
     result_t create_record_type(const hilti::ID& ns, const hilti::ID& local,
                                 const std::vector<hilti::Expression>& fields) {
         if ( hilti::logger().isEnabled(ZeekPlugin) ) {
@@ -1386,43 +1396,55 @@ struct VisitorZeekType : hilti::visitor::PreOrder<hilti::Result<hilti::Expressio
                 SPICY_DEBUG(hilti::util::fmt("Creating (empty) Zeek record type %s::%s", ns, local));
         }
 
-        return builder::call("zeek_rt::create_record_type",
-                             {builder::string(ns), builder::string(local), builder::vector(fields)});
+        auto tmp =
+            body->addTmp(tmpName("fields", {ns, local}), builder::vector(builder::typeByID("zeek_rt::RecordField")));
+
+        for ( const auto& f : fields )
+            body->addMemberCall(tmp, "push_back", {f});
+
+        return builder::call("zeek_rt::create_record_type", {builder::string_mut(ns), builder::string_mut(local), tmp});
     }
 
     hilti::Expression create_record_field(const hilti::ID& id, const hilti::Expression& type, bool optional,
                                           bool log) const {
         return builder::call("zeek_rt::create_record_field",
-                             {builder::string(id), type, builder::bool_(optional), builder::bool_(log)});
+                             {builder::string_mut(id), type, builder::bool_(optional), builder::bool_(log)});
     }
 
     result_t base_type(const char* tag) { return builder::call("zeek_rt::create_base_type", {builder::id(tag)}); }
 
-    result_t createZeekType(const hilti::Type& t, const std::optional<hilti::ID>& id_ = {}) {
-        if ( id_ )
-            ids.push_back(id_);
-        else if ( auto x = t.typeID() )
-            ids.push_back(*x);
-        else
-            ids.push_back(std::nullopt);
+    result_t createZeekType(const hilti::Type& t, std::optional<hilti::ID> id = {}) {
+        if ( ! id )
+            id = t.typeID(); // may still be unset
+
+        if ( id ) {
+            if ( auto x = cache->find(*id); x != cache->end() )
+                return x->second;
 
-        if ( id() ) {
             // Avoid infinite recursion.
-            if ( zeek_types.count(*id()) )
+            if ( zeek_types.count(*id) )
                 return hilti::result::Error("type is self-recursive");
 
-            zeek_types.insert(*id());
+            zeek_types.insert(*id);
         }
 
+        ids.push_back(id);
         auto x = dispatch(t);
+        ids.pop_back();
+
         if ( ! x )
             return hilti::result::Error(
                 hilti::util::fmt("no support for automatic conversion into a Zeek type (%s)", t.typename_()));
 
-        if ( id() )
-            zeek_types.erase(*id());
+        if ( id ) {
+            zeek_types.erase(*id);
 
-        ids.pop_back();
+            if ( *x ) {
+                auto zt = body->addTmp(tmpName("type", *id), **x);
+                cache->emplace(*id, zt);
+                return zt;
+            }
+        }
 
         return *x;
     }
@@ -1474,11 +1496,20 @@ struct VisitorZeekType : hilti::visitor::PreOrder<hilti::Result<hilti::Expressio
         assert(id());
 
         auto labels = hilti::rt::transform(t.labels(), [](const auto& l) {
-            return builder::tuple({builder::string(l.get().id()), builder::integer(l.get().value())});
+            return builder::tuple({builder::string_literal(l.get().id().str()), builder::integer(l.get().value())});
         });
 
-        return builder::call("zeek_rt::create_enum_type", {builder::string(id()->namespace_()),
-                                                           builder::string(id()->local()), builder::vector(labels)});
+        auto tmp = body->addTmp(tmpName("labels", *id()),
+                                hilti::type::Vector(
+                                    hilti::type::Tuple({hilti::type::String(), hilti::type::SignedInteger(64)})));
+
+        for ( const auto& l : t.labels() )
+            body->addMemberCall(tmp, "push_back",
+                                {builder::tuple(
+                                    {builder::string_literal(l.get().id().str()), builder::integer(l.get().value())})});
+
+        return builder::call("zeek_rt::create_enum_type",
+                             {builder::string_mut(id()->namespace_()), builder::string_mut(id()->local()), tmp});
     }
 
     result_t operator()(const hilti::type::Map& t) {
@@ -1594,9 +1625,20 @@ struct VisitorZeekType : hilti::visitor::PreOrder<hilti::Result<hilti::Expressio
 };
 } // namespace
 
-hilti::Result<hilti::Expression> GlueCompiler::createZeekType(const hilti::Type& t, const hilti::ID& id) const {
-    auto v = VisitorZeekType(this);
-    return v.createZeekType(t, id);
+hilti::Result<hilti::Nothing> GlueCompiler::createZeekType(const hilti::Type& t, const hilti::ID& id,
+                                                           hilti::builder::Builder* body,
+                                                           GlueCompiler::ZeekTypeCache* cache) const {
+    body->addComment(hilti::util::fmt("Creating Zeek type %s", id));
+
+    auto v = VisitorZeekType(this, body, cache);
+
+    if ( auto zt = v.createZeekType(t, id) ) {
+        body->addCall("zeek_rt::register_type",
+                      {builder::string_mut(id.namespace_()), builder::string_mut(id.local()), *zt});
+        return hilti::Nothing();
+    }
+    else
+        return zt.error();
 }
 
 namespace {
